/**
   A simple generic implementation of the Bag abstract data type
   that uses an array based implementation with an iterator.

   Here is the public Bag interface.
      public boolean remove(T element);
      public int size();
      public boolean isEmpty();
      public void add(T element);
      public void add(T... elements);
      public void addAll(Container c);
      public boolean contains(T element);
      public int countOccurrences(T element);
*/
import java.util.Iterator;
import java.util.NoSuchElementException;

public class ArrayBag<T> extends Bag<T>
{
   /**
      This is an implementation of the Iterator interface
      that knows how to iterator over an ArrayBag.

      Since the ArraysBag class does not expose any information
      about its internal implementation, this Iterator must
      be a nested class within the ArrayBag class so that the
      iterator object has access to the internals of a ArrayBag
      object.
   */
   class ArrayBagIterator implements Iterator<T>
   {
      int cursor;

      public ArrayBagIterator( )
      {
         this.cursor = 0;
      }

      public boolean hasNext()
      {
         return ( cursor < size );
      }

      @SuppressWarnings("unchecked")
      public T next()
      {
         // Here is where we make use of the fact that
         // this iterator is nested inside of the ArrayList
         // class. We have access to the private array
         // data.
         if ( hasNext() )
         {
            return (T)data[cursor++];
         }
         else
            throw new NoSuchElementException();
      }

      public void remove()
      {
         throw new UnsupportedOperationException();
      }
   }//ArrayBagIterator


   // A private field in the ArrayBag class.
   private Object[] data;
   private final int INITIAL_CAPACITY = 100;

   public ArrayBag()
   {
      super();
      data = new Object[INITIAL_CAPACITY];
   }

   public void add(T element)
   {
      ensureCapacity();
      this.data[size] = element;
      size++;
   }

   public boolean remove(T element)
   {
      boolean found = false;
      for (int i = 0; i < size; i++)
      {
         if ( element.equals(data[i]) )
         {
            data[i] = data[size-1];
            size--;
            found = true;
            break;
         }
      }
      return found;
   }

   private void ensureCapacity()
   {
      if (size == data.length)
         throw new Error("full array");
   }


   // Provide an implementation of the Iterable interface.
   public Iterator<T> iterator()
   {
      return new ArrayBagIterator();
   }

}//ArrayBag
