/**
   A simple generic implementation of the List abstract data type
   that uses an array based implementation with an iterator.

   Here is the public List interface.
      public T getFirst();
      public T remove();
      public int size();
      public boolean isEmpty();
      public void add(T element);
      public void add(T... elements);
      public void addAll(Container c);
      public boolean contains(T element);
      public int countOccurrences(T element);
*/
import java.util.Iterator;
import java.util.NoSuchElementException;

public class ArrayList<T> extends List<T>
{
   /**
      This is an implementation of the Iterator interface
      that knows how to iterator over an ArrayList.

      Since the ArrayList class does not expose any information
      about its internal implementation, this Iterator must
      be a nested class within the ArrayList class so that the
      iterator object has access to the internals of a ArrayList
      object.
   */
   class ArrayListIterator implements Iterator<T>
   {
      int cursor;

      public ArrayListIterator( )
      {
         this.cursor = size - 1;
      }

      public boolean hasNext()
      {
         return (0 < cursor);
      }

      @SuppressWarnings("unchecked")
      public T next()
      {
         // Here is where we make use of the fact that
         // this iterator is nested inside of the ArrayList
         // class. We have access to the private array
         // data.
         if ( hasNext() )
         {
            return (T)data[cursor--];
         }
         else
            throw new NoSuchElementException();
      }

      public void remove()
      {
         throw new UnsupportedOperationException();
      }
   }//ArrayListIterator


   // A private field in the ArrayList class.
   private Object[] data;
   private final int INITIAL_CAPACITY = 100;

   public ArrayList()
   {
      super();
      data = new Object[INITIAL_CAPACITY];
   }

   @SuppressWarnings("unchecked")
   public T getFirst()
   {
      return (T)data[size - 1];
   }

   public void add(T element)
   {
      ensureCapacity();
      this.data[size] = element;
      size++;
   }

   @SuppressWarnings("unchecked")
   public T remove()
   {
      T temp = (T)data[size - 1];
      size--;
      return temp;
   }

   private void ensureCapacity()
   {
      if (size == data.length)
         throw new Error("full array");
   }


   // Provide an implementation of the Iterable interface.
   public Iterator<T> iterator()
   {
      return new ArrayListIterator();
   }

}//ArrayList
