/**
   A simple generic implementation of the Stack abstract data type
   that uses an array based implementation with an iterator.

   See pages 316 - 318 of the textbook.

   Here is what the Stack interface adds to the Container interface.
      public void push(T element);
      public T pop();
      public T peek();
*/
import java.util.Iterator;
import java.util.NoSuchElementException;

public class ArrayStack<T> extends Stack<T>
{
   /**
      This is an implementation of the Iterator interface
      that knows how to iterator over an ArrayStack.

      Since the ArrayStack class does not expose any information
      about its internal implementation, this Iterator must
      be a nested class within the ArrayStack class so that the
      iterator object has access to the internals of a ArrayStack
      object.
   */
   class ArrayStackIterator implements Iterator<T>
   {
      int cursor;

      public ArrayStackIterator( )
      {
         this.cursor = size - 1;
      }

      public boolean hasNext()
      {
         return (0 < cursor);
      }

      @SuppressWarnings("unchecked")
      public T next()
      {
         // Here is where we make use of the fact that
         // this iterator is nested inside of the ArrayStack
         // class. We have access to the private array
         // data.
         if ( hasNext() )
         {
            return (T)data[cursor--];
         }
         else
            throw new NoSuchElementException();
      }

      public void remove()
      {
         throw new UnsupportedOperationException();
      }
   }//ArrayStackIterator


   // A private field in the ArrayStack class.
   private Object[] data;
   private final int INITIAL_CAPACITY = 100;


   public ArrayStack()
   {
      super();
      data = new Object[INITIAL_CAPACITY];
   }


   public void add(T element)
   {
      push(element);
   }


   public void push(T element)
   {
      ensureCapacity();
      this.data[size] = element;
      size++;
   }


   @SuppressWarnings("unchecked")
   public T pop()
   {
      T temp = (T)data[size - 1];
      size--;
      return temp;
   }


   @SuppressWarnings("unchecked")
   public T peek()
   {
      return (T)data[size - 1];
   }


   private void ensureCapacity()
   {
      if (size == data.length)
         throw new Error("full array");
   }

   // Provide an implementation of the Iterable interface.
   public Iterator<T> iterator()
   {
      return new ArrayStackIterator();
   }

}//ArrayStack
