/**
   A simple generic implementation of the Bag abstract data type
   that uses two arrays with an iterator.

   Here is the public Bag interface.
      public boolean remove(T element);
      public int size();
      public boolean isEmpty();
      public void add(T element);
      public void add(T... elements);
      public void addAll(Container c);
      public boolean contains(T element);
      public int countOccurrences(T element);
*/
import java.util.Iterator;
import java.util.NoSuchElementException;

public class ArraysBag<T> extends Bag<T>
{
   /**
      This is an implementation of the Iterator interface
      that knows how to iterator over an ArraysBag.

      Since the ArraysBag class does not expose any information
      about its internal implementation, this Iterator must
      be a nested class within the ArraysBag class so that the
      iterator object has access to the internals of a ArraysBag
      object.
   */
   class ArraysBagIterator implements Iterator<T>
   {
      int cursor;       // idnex into data and dataCounts arrays
      int cursorCount;  // multiplicity counter for current data element
      int elementCount; // counts how many elements have been returned by the iterator

      public ArraysBagIterator( )
      {
         this.cursor = 0;
         this.cursorCount = 0;
         this.elementCount = 0;
      }

      public boolean hasNext()
      {
         return ( this.elementCount < size );
      }

      @SuppressWarnings("unchecked")
      public T next()
      {
         // Here is where we make use of the fact that
         // this iterator is nested inside of the ArrayList
         // class. We have access to the private arrays
         // data and dataCounts.
         if ( hasNext() )
         {
            if ( this.cursorCount < dataCounts[this.cursor] )
            {
               // count one more occurrence of the "current" element
               this.cursorCount++;
            }
            else
            {
               // "move" to the next element
               this.cursor++;
               this.cursorCount = 1;
            }
            this.elementCount++;
            return (T)data[cursor];
         }
         else
            throw new NoSuchElementException();
      }

      public void remove()
      {
         throw new UnsupportedOperationException();
      }
   }//ArraysBagIterator


   // A private field in the ArrayBag class.
   private Object[] data;
   private    int[] dataCounts;
   private int distinctItems;  // the number of entries in data[]
   private final int INITIAL_CAPACITY = 100;

   public ArraysBag()
   {
      super();
      data       = new Object[INITIAL_CAPACITY];
      dataCounts = new int[INITIAL_CAPACITY];
      distinctItems = 0;
   }

   public void add(T element)
   {
      // look for element in the data array
      int index = 0;
      for (; index < distinctItems; index++)
         if (element == data[index])
            break;

      if (index == distinctItems)
      {  // element is not in the bag
         if (distinctItems == data.length)
         {
            ensureCapacity();
         }
         data[distinctItems] = element;
         dataCounts[distinctItems] = 1;
         distinctItems++;
      }
      else
      {  // element is already in the bag
         dataCounts[index] += 1;
      }
      size++;
   }

   public boolean remove(T element)
   {
      // look for element in the data array
      int index = 0;
      for (; index < distinctItems; index++)
         if (element == data[index])
            break;

      if (index == distinctItems)
      {  // element is not in the bag
         return false;
      }
      else
      {  // element is in the bag
         size--;
         dataCounts[index] -= 1;
         if ( 0 == dataCounts[index] )
         {
            data[index]       = data[distinctItems - 1];
            dataCounts[index] = dataCounts[distinctItems - 1];
            distinctItems--;
         }
         return true;
      }
   }

   /**
      For the ArraysBag class, we can provide a much faster
      implementation of countOccurrences() than the one
      provided by the Bag base class.
   */
   @Override
   public int countOccurrences(T element)
   {
      for (int index = 0; index < distinctItems; index++)
      {
         if ( element.equals(data[index]) )
            return dataCounts[index];
      }
      return 0;
   }


   private void ensureCapacity()
   {
      if (size == data.length)
         throw new Error("full array");
   }

   // Provide an implementation of the Iterable interface.
   public Iterator<T> iterator()
   {
      return new ArraysBagIterator();
   }

}//ArraysBag
