/**
   A generic Container abstract data type.

   See pages 300-301 in Chapter 5 of our textbook
   and also see the Java Collection interface.
      https://docs.oracle.com/javase/8/docs/api/java/util/Collection.html

   Here is the public Container interface.
      public int size();
      public boolean isEmpty();
      public void add(T element);
      public void add(T... elements);
      public void addAll(Container c);
      public boolean contains(T element);
      public int countOccurrences(T element);

   In addition, this class provides two static methods
   for finding the maximu element in a container of
   comparable items.
      public static <T extends Comparable<T>> T max(Iterable<T> container)
      public static <T> T max(Iterable<T> container, Comparator<T> cmp)
*/
import java.util.Iterator;
import java.util.Comparator;

public abstract class Container<T> implements Iterable<T>
{
   protected int size;

   // Used in the toString() method.
   private String  OPEN_BRACKET;
   private String CLOSE_BRACKET;

   protected Container(String open, String close)
   {
      size = 0;

      // Used in the toString() method.
      CLOSE_BRACKET = close;
       OPEN_BRACKET = open;
   }

   public int size()
   {
      return size;
   }

   public boolean isEmpty()
   {
      return ( 0 == size() );
   }

   public abstract void add(T element);

   public void add(T... elements)
   {
      for (T e : elements)
         add(e);
   }

   public void addAll(Container<T> c)
   {
      for (T e : c)
         add(e);
   }

   public boolean contains(T element)
   {
      boolean result = false;
      for (T e : this)
      {
         if ( e.equals(element) )
         {
            result = true;
            break;
         }
      }
      return result;
   }

   public int countOccurrences(T element)
   {
      int count = 0;
      for (T e : this)
      {
         if ( e.equals(element) )
         {
            count++;
         }
      }
      return count;
   }


   @Override
   public String toString()
   {
      String result = OPEN_BRACKET + " ";
      // use the iterator
      for (T e : this)
      {
         result += e + " ";
      }
      result += " " + CLOSE_BRACKET;
      return result;
   }


   // Provide an abstract "implementation" of the Iterable interface.
   public abstract Iterator<T> iterator();


   /**
      Find the largest element of any (nonempty)
      iterable container of comparable elements.
   */
   public static <T extends Comparable<T>> T max(Iterable<T> container)
   {
      Iterator<T> it = container.iterator();
      // Assume that the container is not empty.
      T largest = it.next();

      while( it.hasNext() )
      {
         T temp = it.next();
         if ( temp.compareTo( largest ) > 0 )
         {
            largest = temp;
         }
      }
      return largest;
   }



   /**
      Find the largest element of any (nonempty)
      iterable container of elements for which
      there is a comparator.
   */
   public static <T> T max(Iterable<T> container, Comparator<T> cmp)
   {
      Iterator<T> it = container.iterator();
      // Assume that the container is not empty.
      T largest = it.next();

      while( it.hasNext() )
      {
         T temp = it.next();
         if ( cmp.compare( temp, largest ) > 0 )
         {
            largest = temp;
         }
      }
      return largest;
   }

}//Container
